@@ -5,7 +5,7 @@ from openpyxl.styles import Font, PatternFill, Alignment
 from openpyxl.styles.borders import Border, Side, BORDER_THIN, BORDER_MEDIUM, BORDER_THICK
 import pandas as pd
 from natsort import natsorted
-import os
+import os, sys
 from time import sleep
 from tqdm import tqdm
 import warnings
@@ -51,6 +51,20 @@ def clear():
     os.system('cls' if os.name=='nt' else 'clear')
 
 
+# Gives the Passed Arguments
+def getArg(usagestr='', customarg=''): # Gives arguments & Takes Single Custom Arg. optionally
+    args = sys.argv[1:] # Exclude Filename from obtained Arguments list
+    if len(args) < 1:
+        print(usagestr)
+        return {}
+    else:
+        if len(args) > 2:
+            if customarg:
+                if args[1] == customarg or args[1] == '--'+customarg or args[1] == '-'+customarg :
+                    return { 'main': args[0], customarg : args[2] }    
+        else:
+            return { 'main': args[0] }
+
 
 pval='' ; pltseq=[]
 def findMissingPlots(subp): # Takes Subplots of a Plot
@@ -209,7 +223,7 @@ def getFilename(FILE_NAME, noext=0):
 
   
 def createEXCEL(filename, checkfile=0 , autoOpen=1, selectedPlots=0):
-    global plotsDF, startRow
+    global plotsDF, startRow, FileFullPath
     wb=openpyxl.Workbook()
     sheet=wb.active
     sheet.title =  'Final'
@@ -321,11 +335,11 @@ def createEXCEL(filename, checkfile=0 , autoOpen=1, selectedPlots=0):
                 sheet['D'+str(cn)].alignment = alignment; sheet['E'+str(cn)].alignment = alignment
                 cn+=1
 
-    wb.save(filename)  
+    wb.save(FileFullPath+filename)  
             
     if autoOpen :
         sleep(4)
-        os.system('start EXCEL.EXE "'+filename+'"')
+        os.system('start EXCEL.EXE "'+FileFullPath+filename+'"')
 
 
 def selectplots(plotsDF):
@@ -339,9 +353,9 @@ def selectplots(plotsDF):
     tplot=int(tplot)
     def SelectedPlots(i):
         startRange=(i-i%10  if 0<i%10<6 else  i-i%5) if i%10 else i-5
-        return [ startRange+i for i in range(1,6) ]    
-    
-    rtplot=tplot+abs(tplot%10-10)
+        return [ startRange+i for i in range(1,6) ]
+
+    rtplot=tplot+abs(tplot%5-5)
     splotinterval=int(rtplot/4)
     stmflag=True; grpflag=True
     print('\n\n')
@@ -350,18 +364,26 @@ def selectplots(plotsDF):
             stm=input('\n  - Enter Stratum : ')
             stmflag=False if stm.isdigit() else True
         while(grpflag):
-            grp=input('  - Enter Group   : ')
+            grp=input('  - Enter Order of Selection : ')
             grpflag=False if grp.isdigit() else True
         stmgrp=stm+grp
+        
+    if __name__ == '__main__' :
+        RandomTable = pd.read_pickle('RandomTable.pkl')
+    else:
+        RandomTable = pd.read_pickle(os.path.dirname(os.path.realpath(__file__))+'\RandomTable.pkl')
+    if int(stmgrp) <=104 :
+        for num in RandomTable[int(stmgrp)]:
+            if int(num) <= tplot:
+                RNUM=int(num)
+                break
+    else:
+        print('\n\n\n     Random Column Out of Range : ',stmgrp,'\n\n\n')
+        sys.exit()
 
-    RandomTable = pd.read_pickle('RandomTable.pkl')
-    for num in RandomTable[int(stmgrp)]:
-        if int(num) <= tplot:
-            RNUM=int(num)
-            break
-    splots = [ RNUM+splotinterval*i  if RNUM+splotinterval*i <= tplot else RNUM+splotinterval*i-tplot  for i in range(1,5) ] # Four Randomly Selected Plot SNo.
+    splots = [ RNUM+splotinterval*i  if RNUM+splotinterval*i <= tplot else (RNUM+splotinterval*i)-rtplot  for i in range(4) ] # Four Randomly Selected Plot SNo.
 
-    print('\n\n   Random Number : ',RNUM,'\n   Plot Interval : ',splotinterval)
+    print('\n\n   Random Number :',RNUM,'\n\n    H :',tplot," H' :",rtplot,'\n   Plot Interval : ',splotinterval)
     selectedplots={}
     if isinstance(plotsDF,pd.core.frame.DataFrame):
         sleep(2)
@@ -392,17 +414,21 @@ def selectplots(plotsDF):
                 else:
                     spltstr.append('  {:>2}'.format(n)+'.  {:>5}'.format(sno))
                 spltstr.append('\n') if not n%5 else ''
-        return { 'param':[ RNUM,splotinterval ] , 'splots': spltstr }
+        return { 'param':[ RNUM,rtplot,splotinterval ] , 'splots': spltstr }
 
         
 
-
+FileFullPath=''
 def agriplot(FILE_NAME, checkfile=''):
     '''
     Generates Excel file from Plot data file.
     Optionally, Pass the Excel file having manually done Plot sequences in the parameter 'check=' to check with the generated sequences.
     '''
-    wb = openpyxl.load_workbook(getFilename(FILE_NAME))
+    #print(vars(sys.modules[__name__])['__package__'])
+    global FileFullPath
+    FILENAME=getFilename(FILE_NAME)
+    FileFullPath=os.path.abspath(FILENAME).replace(FILENAME,'')
+    wb = openpyxl.load_workbook(getFilename(FILENAME))
     sheet=wb[wb.sheetnames[-1]]
     sheet=wb.active
     global startRow
@@ -477,11 +503,58 @@ def agriplot(FILE_NAME, checkfile=''):
     return plotsDF
 
 
+def agriplotcmd():
+    Usgstr='''
+Usage : Generates Excel file from Plot Data file
+
+        agriplotcmd   FILENAME
+        
+        Optional : Use option 'checkfile' To Compare with a File
+        e.g.  agriplotcmd   FILENAME   --checkfile  FILENAMEToCompare
+        
+        '''
+    argDict=getArg(usagestr=Usgstr, customarg='checkfile')
+    if argDict:
+        FILE_NAME=argDict['main']
+        checkfile=argDict['checkfile']  if argDict.get('checkfile') else ''
+        agriplot(FILE_NAME, checkfile=checkfile)
+
+
+def selectplotscmd():
+    Usgstr='''
+Usage : Takes 'Total Number of Plots/Last SNo.' and Gives the Selected Plots
+
+        selectplotscmd  Number-of-Plots/Last-SNo.
+
+        '''
+    argDict=getArg(usagestr=Usgstr, customarg='checkfile')
+    if argDict:
+        plotsDF=argDict['main']
+        selectedplotdict=selectplots(plotsDF)
+        selectedplots=selectedplotdict['splots']
+        print('\n\n   Selected Plot SNo.\n\n  '+'\n  '.join([ i for i in selectedplots ]))
+
+
+
+def getsubplotscmd():
+    Usgstr='''
+Usage : Pass the Subplots of a Plot separated by comma in any order to get All the Possible Subplots in Sequence
+
+        getsubplotscmd   Comma-Separated-SubPlots
+        '''
+    argDict=getArg(usagestr=Usgstr, customarg='checkfile')
+    if argDict:
+        subplots=argDict['main'].replace('[','').replace(']','')
+        splots=getsubplots(subplots)
+        print('\n   SubPlots:: \n\n        '+'\n        '.join([ i  for i in splots ])+'\n\n')
+
+
+
 if __name__ == '__main__' :
 
     #FILE_NAME='tplots.xlsx'
-    FILE_NAME='data'
+    #FILE_NAME='plotdata'
     #FILE_NAME='plot1.xlsx'
     #agriplot(FILE_NAME, checkfile='done' )
-    agriplot(FILE_NAME )
+    #agriplot(FILE_NAME )
     
